@@ -12,7 +12,7 @@
 //  -------------------------------------------------------------
 """
 
-from logging import getLogger, DEBUG, Formatter, Handler, addLevelName
+from logging import getLogger, DEBUG, Formatter, Handler, addLevelName, NullHandler
 from logging.handlers import RotatingFileHandler
 from sys import platform, version_info
 from sys import argv as sargv
@@ -21,6 +21,7 @@ from time import time, sleep
 from argparse import ArgumentParser, SUPPRESS, Namespace
 from json import load, dumps, loads
 from uuid import uuid4
+from collections.abc import Mapping
 from functools import wraps
 from re import sub as resub
 from re import findall, IGNORECASE
@@ -33,11 +34,12 @@ from tempfile import mkdtemp
 from urllib.parse import unquote, urlparse
 from urllib3.exceptions import InsecureRequestWarning
 from bs4 import BeautifulSoup
-from tld import get_fld
+from tld import get_fld, get_tld
 from requests import get, packages, Session
 from termcolor import colored
 from langdetect import detect
 from warnings import filterwarnings
+from galeodes import Galeodes
 
 filterwarnings('ignore', category=RuntimeWarning, module='runpy')
 packages.urllib3.disable_warnings(category=InsecureRequestWarning)
@@ -66,7 +68,8 @@ class SocialAnalyzer():
         self.ret = False
         self.headers = {"User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:86.0) Gecko/20100101 Firefox/86.0", }
         self.silent = silent
-        self.init_logic()
+        self.screenshots = None
+        self.screenshots_location = None
 
     def delete_keys(self, in_object, keys):
         '''
@@ -130,7 +133,7 @@ class SocialAnalyzer():
                         return func(*args, **kwargs)
                     except Exception as err:
                         pass
-                        # self.print_wrapper(e)
+                        # if not self.silent: self.log.info(e)
                 else:
                     return func(*args, **kwargs)
             return wrapper
@@ -161,45 +164,57 @@ class SocialAnalyzer():
                 '''
 
                 if self.argv.output != "json" and self.sa_object.silent == False:
-                    if record.levelname == "CUSTOM":
-                        for item in record.msg:
-                            with suppress(Exception):
-                                if item == record.msg[0]:
+                    if isinstance(record.msg, Mapping):
+                        if "custom" in record.msg:
+                            for item in record.msg["custom"]:
+                                with suppress(Exception):
+                                    if item == record.msg["custom"][0]:
+                                        print("-----------------------")
+                                    for key, value in item.items():
+                                        if key == "metadata" or key == "extracted":
+                                            if (self.argv.metadata and key == "metadata") or (self.argv.extract and key == "extracted"):
+                                                with suppress(Exception):
+                                                    for idx, _item in enumerate(value):
+                                                        empty_string = key + " " + str(idx)
+                                                        empty_string = colored(empty_string.ljust(13, ' '), 'blue') + ": "
+                                                        for _item_key, _item_value in _item.items():
+                                                            if self.argv.trim and _item_key == "content" and len(_item_value) > 50:
+                                                                empty_string += "{} : {} ".format(colored(_item_key, 'blue'), colored(_item_value[:50] + "..", 'yellow'))
+                                                            else:
+                                                                empty_string += "{} : {} ".format(colored(_item_key, 'blue'), colored(_item_value, 'yellow'))
+                                                        print("{}".format(empty_string))
+                                        else:
+                                            print(colored(key.ljust(13, ' '), 'blue'), colored(value, 'yellow'), sep=": ")
                                     print("-----------------------")
-                                for key, value in item.items():
-                                    if key == "metadata" or key == "extracted":
-                                        if (self.argv.metadata and key == "metadata") or (self.argv.extract and key == "extracted"):
-                                            with suppress(Exception):
-                                                for idx, _item in enumerate(value):
-                                                    empty_string = key + " " + str(idx)
-                                                    empty_string = colored(empty_string.ljust(13, ' '), 'blue') + ": "
-                                                    for _item_key, _item_value in _item.items():
-                                                        if self.argv.trim and _item_key == "content" and len(_item_value) > 50:
-                                                            empty_string += "{} : {} ".format(colored(_item_key, 'blue'), colored(_item_value[:50] + "..", 'yellow'))
-                                                        else:
-                                                            empty_string += "{} : {} ".format(colored(_item_key, 'blue'), colored(_item_value, 'yellow'))
-                                                    print("{}".format(empty_string))
-                                    else:
-                                        print(colored(key.ljust(13, ' '), 'blue'), colored(value, 'yellow'), sep=": ")
-                                print("-----------------------")
                     else:
                         print(record.msg)
 
+        temp_folder = ''
         if argv.logs:
             if self.logs_dir != '':
                 temp_folder = self.logs_dir
             else:
                 temp_folder = mkdtemp()
-            if argv.output != "json":
-                self.print_wrapper('[init] Temporary Logs Directory {}'.format(temp_folder))
+
             if file and uuid:
-                fh = RotatingFileHandler(path.join(temp_folder, uuid))
+                if argv.screenshots:
+                    self.screenshots = True
+                makedirs(path.join(temp_folder, uuid), exist_ok=True)
+                self.screenshots_location = path.join(temp_folder, uuid)
+                fh = RotatingFileHandler(path.join(temp_folder, uuid, 'logs'))
                 fh.setFormatter(Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s"))
                 self.log.addHandler(fh)
 
         self.log.setLevel(DEBUG)
-        self.log.addHandler(CustomHandler(argv, sa_object=self))
-        addLevelName(self.custom_message, "CUSTOM")
+
+        if argv.silent:
+            self.log.addHandler(NullHandler())
+        else:
+            self.log.addHandler(CustomHandler(argv, sa_object=self))
+
+        if argv.logs and argv.output != "json":
+            if not self.silent:
+                self.log.info('[init] Temporary Logs Directory {}'.format(temp_folder))
 
     def init_detections(self, detections):
         '''
@@ -247,7 +262,8 @@ class SocialAnalyzer():
             for site in self.websites_entries:
                 temp_value = get_fld(site["url"], fix_protocol=True)
                 temp_value = temp_value.replace(".{username}", "").replace("{username}.", "")
-                self.log.info(temp_value)
+                if not self.silent:
+                    self.log.info(temp_value)
 
     def fetch_url(self, site, username, options):
         '''
@@ -259,7 +275,15 @@ class SocialAnalyzer():
         else:
             sleep(randint(1, 99) / 100)
 
-        self.log.info("[Checking] " + get_fld(site["url"]))
+        checking_url = None
+        with suppress(Exception):
+            checking_url = get_tld(site["url"], as_object=True).parsed_url.netloc
+        if checking_url is None:
+            checking_url = get_fld(site["url"])
+        checking_url = checking_url.replace(".{username}", "").replace("{username}.", "")
+        if not self.silent:
+            self.log.info("[Checking] " + checking_url)
+
         source = ""
 
         detection_level = {
@@ -532,8 +556,17 @@ class SocialAnalyzer():
         for i in range(3):
             self.websites_entries[:] = [d for d in self.websites_entries if d.get('selected') == "true"]
             if len(self.websites_entries) > 0:
+                if len(req["body"]["string"].split(',')) > 1:
+                    if not self.silent:
+                        self.log.info("[Info] usernames: {}".format(", ".join(req["body"]["string"].split(','))))
+                else:
+                    if not self.silent:
+                        self.log.info("[Info] username: {}".format(req["body"]["string"]))
                 with ThreadPoolExecutor(max_workers=self.workers) as executor:
-                    future_fetch_url = (executor.submit(self.fetch_url, site, req["body"]["string"], req["body"]["options"]) for site in self.websites_entries)
+                    future_fetch_url = []
+                    for site in self.websites_entries:
+                        for username in req["body"]["string"].split(','):
+                            future_fetch_url.append(executor.submit(self.fetch_url, site, username, req["body"]["options"]))
                     for future in as_completed(future_fetch_url):
                         with suppress(Exception):
                             good, site, data = future.result()
@@ -541,7 +574,8 @@ class SocialAnalyzer():
                                 self.websites_entries[:] = [d for d in self.websites_entries if d.get('url') != site]
                                 resutls.append(data)
                             else:
-                                self.log.info("[Waiting to retry] " + self.get_website(site))
+                                if not self.silent:
+                                    self.log.info("[Waiting to retry] " + self.get_website(site))
 
         self.websites_entries[:] = [d for d in self.websites_entries if d.get('selected') == "true"]
         if len(self.websites_entries) > 0:
@@ -567,16 +601,18 @@ class SocialAnalyzer():
 
         req = {"body": {"uuid": str(uuid4()), "string": argv.username, "options": temp_options}}
         self.setup_logger(uuid=req["body"]["uuid"], file=True, argv=argv)
+        self.init_logic()
 
         if argv.cli:
-            self.log.info("[Warning] --cli is not needed and will be removed later on")
+            if not self.silent:
+                self.log.info("[Warning] --cli is not needed and will be removed later on")
 
         for site in self.websites_entries:
             site["selected"] = "false"
 
         if argv.websites == "all":
             list_of_countries = []
-            if argv.countries != 'all':
+            if argv.countries != "all":
                 list_of_countries = argv.countries.split(" ")
                 for site in self.websites_entries:
                     if site["country"] != "" and site["country"].lower() in list_of_countries:
@@ -587,6 +623,18 @@ class SocialAnalyzer():
                 for site in self.websites_entries:
                     site["selected"] = "true"
 
+            if argv.type != "all":
+                sites = ([d for d in self.websites_entries if d.get('selected') == "true"])
+                if "adult" in argv.type.lower():
+                    for site in sites:
+                        if "adult" in site["type"].lower():
+                            self.search_and_change(site, {"selected": "pendding"})
+                for site in self.websites_entries:
+                    if site["selected"] == "pendding":
+                        site["selected"] = "true"
+                    else:
+                        site["selected"] = "false"
+
             if int(argv.top) != 0:
                 sites = ([d for d in self.websites_entries if d.get('selected') == "true"])
                 sites = ([d for d in sites if d.get('global_rank') != 0])
@@ -609,7 +657,8 @@ class SocialAnalyzer():
             if site["selected"] == "true":
                 true_websites += 1
 
-        self.log.info("[Init] Selected websites: {}".format(true_websites))
+        if not self.silent:
+            self.log.info("[Init] Selected websites: {}".format(true_websites))
         resutls = self.find_username_normal(req)
 
         for item in resutls:
@@ -671,42 +720,74 @@ class SocialAnalyzer():
 
         if argv.output == "pretty" or argv.output == "":
             if 'detected' in temp_detected:
-                self.log.info("[Detected] {} Profile[s]".format(len(temp_detected['detected'])))
-                self.log.log(self.custom_message, temp_detected['detected'])
+                if not self.silent:
+                    self.log.info("[Detected] {} Profile[s]".format(len(temp_detected['detected'])))
             if 'unknown' in temp_detected:
-                self.log.info("[unknown] {} Profile[s]".format(len(temp_detected['unknown'])))
-                self.log.log(self.custom_message, temp_detected['unknown'])
+                if not self.silent:
+                    self.log.info("[unknown] {} Profile[s]".format(len(temp_detected['unknown'])))
             if 'failed' in temp_detected:
-                self.log.info("[failed] {} Profile[s]".format(len(temp_detected['failed'])))
-                self.log.log(self.custom_message, temp_detected['failed'])
+                if not self.silent:
+                    self.log.info("[failed] {} Profile[s]".format(len(temp_detected['failed'])))
+
+        if "detected" in temp_detected:
+            if self.screenshots and self.screenshots_location:
+                location = None
+                with suppress(Exception):
+                    if not self.silent:
+                        self.log.info("[Info] Getting screenshots of {} profiles".format(len([item['link'] for item in temp_detected["detected"]])))
+                with suppress(Exception):
+                    g = Galeodes(browser="chrome", arguments=['--headless', self.headers['User-Agent']], options=None, implicit_wait=5, verbose=False)
+                    results = g.get_pages(urls=[item['link'] for item in temp_detected["detected"]], screenshots=True, number_of_workers=10, format='jpeg', base64=False)
+                    for item in results:
+                        if item['image'] is not None:
+                            with suppress(Exception):
+                                file_name = resub(r'[^\w\d-]', '_', item['url']) + '.jpeg'
+                                with open(path.join(self.screenshots_location, file_name), 'wb') as f:
+                                    f.write(item['image'])
+                                    location = self.screenshots_location
+                if location:
+                    if not self.silent:
+                        self.log.info("[Info] Screenshots location {}".format(location))
+
+        if argv.output == "pretty" or argv.output == "":
+            if 'detected' in temp_detected:
+                if not self.silent:
+                    self.log.info({"custom": temp_detected['detected']})
+            if 'unknown' in temp_detected:
+                if not self.silent:
+                    self.log.info({"custom": temp_detected['unknown']})
+            if 'failed' in temp_detected:
+                if not self.silent:
+                    self.log.info({"custom": temp_detected['failed']})
 
         if argv.output == "json":
-            self.print_wrapper(dumps(temp_detected, sort_keys=True, indent=None))
+            if not self.silent:
+                print(dumps(temp_detected, sort_keys=True, indent=None))
 
         return temp_detected
 
-    def print_wrapper(self, *args, **kwarg):
-        if self.silent == False:
-            print(*args, **kwarg)
-
     def load_file(self, name, path_to_check, url_download):
         ret = None
         try:
             if path.exists(path_to_check) == False:
-                self.print_wrapper("[init] Downloading {} from {}".format(name, url_download))
+                if not self.silent:
+                    self.log.info("[init] Downloading {} from {}".format(name, url_download))
                 file = get(url_download, allow_redirects=True)
                 with open(path_to_check, 'wb') as f:
                     f.write(file.content)
             if path.exists(path_to_check) == True:
-                self.print_wrapper("[init] {} looks good!".format(name))
+                if not self.silent:
+                    self.log.info("[init] {} looks good!".format(name))
                 with open(path_to_check, encoding="utf-8") as f:
                     ret = load(f)
         except Exception as e:
-            self.print_wrapper("[!] {} Does not exist! cannot be downloaded...".format(name))
+            if not self.silent:
+                self.log.info("[!] {} Does not exist! cannot be downloaded...".format(name))
         return ret
 
     def init_logic(self):
-        self.print_wrapper("[init] Detections are updated very often, make sure to get the most up-to-date ones")
+        if not self.silent:
+            self.log.info("[init] Detections are updated very often, make sure to get the most up-to-date ones")
         if platform == "win32":
             system("color")
         makedirs(path.join(path.dirname(__file__), "data"), exist_ok=True)
@@ -716,12 +797,14 @@ class SocialAnalyzer():
         self.shared_detections = self.init_detections("shared_detections")
         self.generic_detection = self.init_detections("generic_detection")
         if self.languages_json is not None and self.sites_dummy is not None:
-            self.print_wrapper("[init] languages.json & sites.json loaded successfully")
+            if not self.silent:
+                self.log.info("[init] languages.json & sites.json loaded successfully")
         else:
-            self.print_wrapper("[init] languages.json & sites.json did not load, exiting..")
+            if not self.silent:
+                self.log.info("[init] languages.json & sites.json did not load, exiting..")
             exit()
 
-    def run_as_object(self, cli=False, gui=False, logs_dir='', logs=False, extract=False, filter='good', headers={}, list=False, metadata=False, method='all', mode='fast', options='', output='pretty', profiles='detected', ret=False, silent=False, timeout=0, trim=False, username='', websites='all', countries='all', top='0'):
+    def run_as_object(self, cli=False, gui=False, logs_dir='', logs=False, extract=False, filter='good', headers={}, list=False, metadata=False, method='all', mode='fast', options='', output='pretty', profiles='detected', type='all', ret=False, silent=False, timeout=0, trim=False, username='', websites='all', countries='all', top='0', screenshots=False):
         ret = {}
         if logs_dir != '':
             self.logs_dir = logs_dir
@@ -738,6 +821,7 @@ class SocialAnalyzer():
         if ARGV.list:
             self.setup_logger(argv=ARGV)
             self.list_all_websites()
+            self.init_logic()
         elif ARGV.mode == "fast":
             if ARGV.username != "" and ARGV.websites != "":
                 ret = self.check_user_cli(ARGV)
@@ -763,12 +847,14 @@ class SocialAnalyzer():
         ARG_PARSER_OPTIONAL.add_argument("--filter", help="Filter detected profiles by good, maybe or bad, you can do combine them with comma (good,bad) or use all", metavar="", default="good")
         ARG_PARSER_OPTIONAL.add_argument("--profiles", help="Filter profiles by detected, unknown or failed, you can do combine them with comma (detected,failed) or use all", metavar="", default="detected")
         ARG_PARSER_OPTIONAL.add_argument("--countries", help="select websites by country or countries separated by space as: us br ru", metavar="", default="all")
+        ARG_PARSER_OPTIONAL.add_argument("--type", help="Select websites by type (Adult, Music etc)", metavar="", default="all")
         ARG_PARSER_OPTIONAL.add_argument("--top", help="select top websites as 10, 50 etc...[--websites is not needed]", metavar="", default="0")
         ARG_PARSER_OPTIONAL.add_argument("--extract", help="Extract profiles, urls & patterns if possible", action="store_true")
         ARG_PARSER_OPTIONAL.add_argument("--metadata", help="Extract metadata if possible (pypi QeeqBox OSINT)", action="store_true")
         ARG_PARSER_OPTIONAL.add_argument("--trim", help="Trim long strings", action="store_true")
         ARG_PARSER_OPTIONAL.add_argument("--gui", help="Reserved for a gui (Not implemented)", action="store_true")
         ARG_PARSER_OPTIONAL.add_argument("--cli", help="Reserved for a cli (Not needed)", action="store_true")
+        ARG_PARSER_OPTIONAL.add_argument("--screenshots", help="Get screenshots from detected profiles (This needs --logs)", action="store_true")
         ARG_PARSER_LIST = ARG_PARSER.add_argument_group("Listing websites & detections")
         ARG_PARSER_LIST.add_argument("--list", help="List all available websites", action="store_true")
         ARG_PARSER_SETTINGS = ARG_PARSER.add_argument_group("Setting")
@@ -777,8 +863,8 @@ class SocialAnalyzer():
         ARG_PARSER_SETTINGS.add_argument("--logs_dir", help="Change logs directory", metavar="", default="")
         ARG_PARSER_SETTINGS.add_argument("--timeout", help="Change timeout between each request", metavar="", type=int, default=0)
         ARG_PARSER_SETTINGS.add_argument("--silent", help="Disable output to screen", action="store_true")
-        ARGV = ARG_PARSER.parse_args()
 
+        ARGV = ARG_PARSER.parse_args()
         if ARGV.logs_dir != '':
             self.logs_dir = ARGV.logs_dir
         if ARGV.headers != {}:
@@ -790,6 +876,7 @@ class SocialAnalyzer():
         if ARGV.list:
             self.setup_logger(argv=ARGV)
             self.list_all_websites()
+            self.init_logic()
         elif ARGV.mode == "fast":
             if ARGV.username != "" and ARGV.websites != "":
                 ret = self.check_user_cli(ARGV)
